@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's parser. */
 
 /*
- *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,16 +21,54 @@
  */
 
 #include "parser.h"
+#include "lst.h"
 #include "lexer.h"
 #include "debug.h"
 #include "mul.h"
 #include "constant.h"
+#include "function.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h> // for uintptr_t
+#endif
 #include <sstream>
 #include <stdexcept>
 
 namespace GiNaC {
 
+// <KLUDGE>
+// Find out if ptr is a pointer to a function or a specially crafted integer.
+// It's possible to distinguish between these because functions are aligned.
+// Returns true if ptr is a pointer and false otherwise.
+static bool decode_serial(unsigned& serial, const reader_func ptr)
+{
+	uintptr_t u = (uintptr_t)(void *)ptr;
+	if (u & 1) {
+		u >>= 1;
+		serial = (unsigned)u;
+		return false;
+	}
+	return true;
+}
+
+// Figures out if ptr is a pointer to function or a serial of GiNaC function.
+// In the former case calls that function, in the latter case constructs
+// GiNaC function with corresponding serial and arguments.
+static ex dispatch_reader_fcn(const reader_func ptr, const exvector& args)
+{
+	unsigned serial = 0; // dear gcc, could you please shut up?
+	bool is_ptr = decode_serial(serial, ptr);
+	if (is_ptr)
+		return ptr(args);
+	else
+		return function(serial, args);
+}
+// </KLUDGE>
+
+
 /// identifier_expr:  identifier |  identifier '(' expression* ')'
 ex parser::parse_identifier_expr()
 {
@@ -65,7 +103,9 @@ ex parser::parse_identifier_expr()
 		Parse_error_("no function \"" << name << "\" with " <<
 			     args.size() << " arguments");
 	}
-	ex ret = reader->second(args);
+	// reader->second might be a pointer to a C++ function or a specially
+	// crafted serial of a GiNaC::function.
+	ex ret = dispatch_reader_fcn(reader->second, args);
 	return ret;
 }
 
@@ -81,8 +121,35 @@ ex parser::parse_paren_expr()
 	return e;
 }
 
-extern numeric* _num_1_p;
-extern ex _ex0;
+/// lst_expr:  '{' expression { ',' expression } '}'
+ex parser::parse_lst_expr()
+{
+	get_next_tok();  // eat {.
+
+	lst list;
+	if (token != '}') {
+		while (true) {
+			ex e = parse_expression(); // expression();
+			list.append(e);
+
+			if (token == '}') {
+				break;
+			}
+
+			if (token != ',') {
+				Parse_error("expected '}'");
+			}
+
+			get_next_tok();  // eat ','.
+		}
+	}
+	// Eat the '}'.
+	get_next_tok();
+
+	return list;
+}
+
+extern const ex _ex0;
 
 /// unary_expr: [+-] expression
 ex parser::parse_unary_expr()
@@ -113,6 +180,8 @@ ex parser::parse_primary()
 			 return parse_number_expr();
 		case '(': 
 			 return parse_paren_expr();
+		case '{': 
+			 return parse_lst_expr();
 		case '-':
 		case '+':
 			 return parse_unary_expr();
