@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's symbolic exponentiation (basis^exponent). */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <vector>
-#include <iostream>
-#include <stdexcept>
-#include <limits>
-
 #include "power.h"
 #include "expairseq.h"
 #include "add.h"
@@ -43,6 +38,12 @@
 #include "relational.h"
 #include "compiler.h"
 
+#include <iostream>
+#include <limits>
+#include <stdexcept>
+#include <vector>
+#include <algorithm>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(power, basic,
@@ -50,15 +51,14 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(power, basic,
   print_func<print_latex>(&power::do_print_latex).
   print_func<print_csrc>(&power::do_print_csrc).
   print_func<print_python>(&power::do_print_python).
-  print_func<print_python_repr>(&power::do_print_python_repr))
-
-typedef std::vector<int> intvector;
+  print_func<print_python_repr>(&power::do_print_python_repr).
+  print_func<print_csrc_cl_N>(&power::do_print_csrc_cl_N))
 
 //////////
 // default constructor
 //////////
 
-power::power() : inherited(&power::tinfo_static) { }
+power::power() { }
 
 //////////
 // other constructors
@@ -70,8 +70,9 @@ power::power() : inherited(&power::tinfo_static) { }
 // archiving
 //////////
 
-power::power(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
+void power::read_archive(const archive_node &n, lst &sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	n.find_ex("basis", basis, sym_lst);
 	n.find_ex("exponent", exponent, sym_lst);
 }
@@ -83,8 +84,6 @@ void power::archive(archive_node &n) const
 	n.add_ex("exponent", exponent);
 }
 
-DEFAULT_UNARCHIVE(power)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -162,26 +161,26 @@ static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 	}
 }
 
-void power::do_print_csrc(const print_csrc & c, unsigned level) const
+void power::do_print_csrc_cl_N(const print_csrc_cl_N& c, unsigned level) const
 {
-	if (is_a<print_csrc_cl_N>(c)) {
-		if (exponent.is_equal(_ex_1)) {
-			c.s << "recip(";
-			basis.print(c);
-			c.s << ')';
-			return;
-		}
-		c.s << "expt(";
+	if (exponent.is_equal(_ex_1)) {
+		c.s << "recip(";
 		basis.print(c);
-		c.s << ", ";
-		exponent.print(c);
 		c.s << ')';
 		return;
 	}
+	c.s << "expt(";
+	basis.print(c);
+	c.s << ", ";
+	exponent.print(c);
+	c.s << ')';
+}
 
+void power::do_print_csrc(const print_csrc & c, unsigned level) const
+{
 	// Integer powers of symbols are printed in a special, optimized way
-	if (exponent.info(info_flags::integer)
-	 && (is_a<symbol>(basis) || is_a<constant>(basis))) {
+	if (exponent.info(info_flags::integer) &&
+	    (is_a<symbol>(basis) || is_a<constant>(basis))) {
 		int exp = ex_to<numeric>(exponent).to_int();
 		if (exp > 0)
 			c.s << '(';
@@ -240,6 +239,11 @@ bool power::info(unsigned inf) const
 			       basis.info(inf);
 		case info_flags::expanded:
 			return (flags & status_flags::expanded);
+		case info_flags::positive:
+			return basis.info(info_flags::positive) && exponent.info(info_flags::real);
+		case info_flags::nonnegative:
+			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
+			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
 		case info_flags::has_indices: {
 			if (flags & status_flags::has_indices)
 				return true;
@@ -285,11 +289,16 @@ ex power::map(map_function & f) const
 
 bool power::is_polynomial(const ex & var) const
 {
-	if (exponent.has(var))
-		return false;
-	if (!exponent.info(info_flags::nonnegint))
-		return false;
-	return basis.is_polynomial(var);
+	if (basis.is_polynomial(var)) {
+		if (basis.has(var))
+			// basis is non-constant polynomial in var
+			return exponent.info(info_flags::nonnegint);
+		else
+			// basis is constant in var
+			return !exponent.has(var);
+	}
+	// basis is a non-polynomial function of var
+	return false;
 }
 
 int power::degree(const ex & s) const
@@ -360,7 +369,7 @@ ex power::coeff(const ex & s, int n) const
  *  - ^(1,x) -> 1
  *  - ^(c1,c2) -> *(c1^n,c1^(c2-n))  (so that 0<(c2-n)<1, try to evaluate roots, possibly in numerator and denominator of c1)
  *  - ^(^(x,c1),c2) -> ^(x,c1*c2)  if x is positive and c1 is real.
- *  - ^(^(x,c1),c2) -> ^(x,c1*c2)  (c2 integer or -1 < c1 <= 1, case c1=1 should not happen, see below!)
+ *  - ^(^(x,c1),c2) -> ^(x,c1*c2)  (c2 integer or -1 < c1 <= 1 or (c1=-1 and c2>0), case c1=1 should not happen, see below!)
  *  - ^(*(x,y,z),c) -> *(x^c,y^c,z^c)  (if c integer)
  *  - ^(*(x,c1),c2) -> ^(x,c2)*c1^c2  (c1>0)
  *  - ^(*(x,c1),c2) -> ^(-x,c2)*c1^c2  (c1<0)
@@ -376,17 +385,13 @@ ex power::eval(int level) const
 	const ex & ebasis    = level==1 ? basis    : basis.eval(level-1);
 	const ex & eexponent = level==1 ? exponent : exponent.eval(level-1);
 	
-	bool basis_is_numerical = false;
-	bool exponent_is_numerical = false;
-	const numeric *num_basis;
-	const numeric *num_exponent;
+	const numeric *num_basis = NULL;
+	const numeric *num_exponent = NULL;
 	
 	if (is_exactly_a<numeric>(ebasis)) {
-		basis_is_numerical = true;
 		num_basis = &ex_to<numeric>(ebasis);
 	}
 	if (is_exactly_a<numeric>(eexponent)) {
-		exponent_is_numerical = true;
 		num_exponent = &ex_to<numeric>(eexponent);
 	}
 	
@@ -403,7 +408,7 @@ ex power::eval(int level) const
 		return ebasis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
-	if (ebasis.is_zero() && exponent_is_numerical) {
+	if ( ebasis.is_zero() && num_exponent ) {
 		if ((num_exponent->real()).is_zero())
 			throw (std::domain_error("power::eval(): pow(0,I) is undefined"));
 		else if ((num_exponent->real()).is_negative())
@@ -424,11 +429,11 @@ ex power::eval(int level) const
 	if (is_exactly_a<power>(ebasis) && ebasis.op(0).info(info_flags::positive) && ebasis.op(1).info(info_flags::real))
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
 
-	if (exponent_is_numerical) {
+	if ( num_exponent ) {
 
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
 		// except if c1,c2 are rational, but c1^c2 is not)
-		if (basis_is_numerical) {
+		if ( num_basis ) {
 			const bool basis_is_crational = num_basis->is_crational();
 			const bool exponent_is_crational = num_exponent->is_crational();
 			if (!basis_is_crational || !exponent_is_crational) {
@@ -482,7 +487,7 @@ ex power::eval(int level) const
 		}
 	
 		// ^(^(x,c1),c2) -> ^(x,c1*c2)
-		// (c1, c2 numeric(), c2 integer or -1 < c1 <= 1,
+		// (c1, c2 numeric(), c2 integer or -1 < c1 <= 1 or (c1=-1 and c2>0),
 		// case c1==1 should not happen, see below!)
 		if (is_exactly_a<power>(ebasis)) {
 			const power & sub_power = ex_to<power>(ebasis);
@@ -491,7 +496,8 @@ ex power::eval(int level) const
 			if (is_exactly_a<numeric>(sub_exponent)) {
 				const numeric & num_sub_exponent = ex_to<numeric>(sub_exponent);
 				GINAC_ASSERT(num_sub_exponent!=numeric(1));
-				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative()) {
+				if (num_exponent->is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative() ||
+				    (num_sub_exponent == *_num_1_p && num_exponent->is_positive())) {
 					return power(sub_basis,num_sub_exponent.mul(*num_exponent));
 				}
 			}
@@ -541,6 +547,7 @@ ex power::eval(int level) const
 					if (num_coeff.is_positive()) {
 						mul *mulp = new mul(mulref);
 						mulp->overall_coeff = _ex1;
+						mulp->setflag(status_flags::dynallocated);
 						mulp->clearflag(status_flags::evaluated);
 						mulp->clearflag(status_flags::hash_calculated);
 						return (new mul(power(*mulp,exponent),
@@ -550,6 +557,7 @@ ex power::eval(int level) const
 						if (!num_coeff.is_equal(*_num_1_p)) {
 							mul *mulp = new mul(mulref);
 							mulp->overall_coeff = _ex_1;
+							mulp->setflag(status_flags::dynallocated);
 							mulp->clearflag(status_flags::evaluated);
 							mulp->clearflag(status_flags::hash_calculated);
 							return (new mul(power(*mulp,exponent),
@@ -615,26 +623,24 @@ bool power::has(const ex & other, unsigned options) const
 		return basic::has(other, options);
 	if (!is_a<power>(other))
 		return basic::has(other, options);
-	if (!exponent.info(info_flags::integer)
-			|| !other.op(1).info(info_flags::integer))
+	if (!exponent.info(info_flags::integer) ||
+	    !other.op(1).info(info_flags::integer))
 		return basic::has(other, options);
-	if (exponent.info(info_flags::posint)
-			&& other.op(1).info(info_flags::posint)
-			&& ex_to<numeric>(exponent).to_int()
-					> ex_to<numeric>(other.op(1)).to_int()
-			&& basis.match(other.op(0)))
+	if (exponent.info(info_flags::posint) &&
+	    other.op(1).info(info_flags::posint) &&
+	    ex_to<numeric>(exponent) > ex_to<numeric>(other.op(1)) &&
+	    basis.match(other.op(0)))
 		return true;
-	if (exponent.info(info_flags::negint)
-			&& other.op(1).info(info_flags::negint)
-			&& ex_to<numeric>(exponent).to_int()
-					< ex_to<numeric>(other.op(1)).to_int()
-			&& basis.match(other.op(0)))
+	if (exponent.info(info_flags::negint) &&
+	    other.op(1).info(info_flags::negint) &&
+	    ex_to<numeric>(exponent) < ex_to<numeric>(other.op(1)) &&
+	    basis.match(other.op(0)))
 		return true;
 	return basic::has(other, options);
 }
 
 // from mul.cpp
-extern bool tryfactsubs(const ex &, const ex &, int &, lst &);
+extern bool tryfactsubs(const ex &, const ex &, int &, exmap&);
 
 ex power::subs(const exmap & m, unsigned options) const
 {	
@@ -650,9 +656,13 @@ ex power::subs(const exmap & m, unsigned options) const
 
 	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
 		int nummatches = std::numeric_limits<int>::max();
-		lst repls;
-		if (tryfactsubs(*this, it->first, nummatches, repls))
-			return (ex_to<basic>((*this) * power(it->second.subs(ex(repls), subs_options::no_pattern) / it->first.subs(ex(repls), subs_options::no_pattern), nummatches))).subs_one_level(m, options);
+		exmap repls;
+		if (tryfactsubs(*this, it->first, nummatches, repls)) {
+			ex anum = it->second.subs(repls, subs_options::no_pattern);
+			ex aden = it->first.subs(repls, subs_options::no_pattern);
+			ex result = (*this)*power(anum/aden, nummatches);
+			return (ex_to<basic>(result)).subs_one_level(m, options);
+		}
 	}
 
 	return subs_one_level(m, options);
@@ -665,66 +675,94 @@ ex power::eval_ncmul(const exvector & v) const
 
 ex power::conjugate() const
 {
-	ex newbasis = basis.conjugate();
-	ex newexponent = exponent.conjugate();
-	if (are_ex_trivially_equal(basis, newbasis) && are_ex_trivially_equal(exponent, newexponent)) {
-		return *this;
+	// conjugate(pow(x,y))==pow(conjugate(x),conjugate(y)) unless on the
+	// branch cut which runs along the negative real axis.
+	if (basis.info(info_flags::positive)) {
+		ex newexponent = exponent.conjugate();
+		if (are_ex_trivially_equal(exponent, newexponent)) {
+			return *this;
+		}
+		return (new power(basis, newexponent))->setflag(status_flags::dynallocated);
 	}
-	return (new power(newbasis, newexponent))->setflag(status_flags::dynallocated);
+	if (exponent.info(info_flags::integer)) {
+		ex newbasis = basis.conjugate();
+		if (are_ex_trivially_equal(basis, newbasis)) {
+			return *this;
+		}
+		return (new power(newbasis, exponent))->setflag(status_flags::dynallocated);
+	}
+	return conjugate_function(*this).hold();
 }
 
 ex power::real_part() const
 {
+	// basis == a+I*b, exponent == c+I*d
+	const ex a = basis.real_part();
+	const ex c = exponent.real_part();
+	if (basis.is_equal(a) && exponent.is_equal(c)) {
+		// Re(a^c)
+		return *this;
+	}
+
+	const ex b = basis.imag_part();
 	if (exponent.info(info_flags::integer)) {
-		ex basis_real = basis.real_part();
-		if (basis_real == basis)
-			return *this;
-		realsymbol a("a"),b("b");
-		ex result;
-		if (exponent.info(info_flags::posint))
-			result = power(a+I*b,exponent);
-		else
-			result = power(a/(a*a+b*b)-I*b/(a*a+b*b),-exponent);
-		result = result.expand();
-		result = result.real_part();
-		result = result.subs(lst( a==basis_real, b==basis.imag_part() ));
+		// Re((a+I*b)^c)  w/  c ∈ ℤ
+		long N = ex_to<numeric>(c).to_long();
+		// Use real terms in Binomial expansion to construct
+		// Re(expand(power(a+I*b, N))).
+		long NN = N > 0 ? N : -N;
+		ex numer = N > 0 ? _ex1 : power(power(a,2) + power(b,2), NN);
+		ex result = 0;
+		for (long n = 0; n <= NN; n += 2) {
+			ex term = binomial(NN, n) * power(a, NN-n) * power(b, n) / numer;
+			if (n % 4 == 0) {
+				result += term;  // sign: I^n w/ n == 4*m
+			} else {
+				result -= term;  // sign: I^n w/ n == 4*m+2
+			}
+		}
 		return result;
 	}
-	
-	ex a = basis.real_part();
-	ex b = basis.imag_part();
-	ex c = exponent.real_part();
-	ex d = exponent.imag_part();
+
+	// Re((a+I*b)^(c+I*d))
+	const ex d = exponent.imag_part();
 	return power(abs(basis),c)*exp(-d*atan2(b,a))*cos(c*atan2(b,a)+d*log(abs(basis)));
 }
 
 ex power::imag_part() const
 {
+	const ex a = basis.real_part();
+	const ex c = exponent.real_part();
+	if (basis.is_equal(a) && exponent.is_equal(c)) {
+		// Im(a^c)
+		return 0;
+	}
+
+	const ex b = basis.imag_part();
 	if (exponent.info(info_flags::integer)) {
-		ex basis_real = basis.real_part();
-		if (basis_real == basis)
-			return 0;
-		realsymbol a("a"),b("b");
-		ex result;
-		if (exponent.info(info_flags::posint))
-			result = power(a+I*b,exponent);
-		else
-			result = power(a/(a*a+b*b)-I*b/(a*a+b*b),-exponent);
-		result = result.expand();
-		result = result.imag_part();
-		result = result.subs(lst( a==basis_real, b==basis.imag_part() ));
+		// Im((a+I*b)^c)  w/  c ∈ ℤ
+		long N = ex_to<numeric>(c).to_long();
+		// Use imaginary terms in Binomial expansion to construct
+		// Im(expand(power(a+I*b, N))).
+		long p = N > 0 ? 1 : 3;  // modulus for positive sign
+		long NN = N > 0 ? N : -N;
+		ex numer = N > 0 ? _ex1 : power(power(a,2) + power(b,2), NN);
+		ex result = 0;
+		for (long n = 1; n <= NN; n += 2) {
+			ex term = binomial(NN, n) * power(a, NN-n) * power(b, n) / numer;
+			if (n % 4 == p) {
+				result += term;  // sign: I^n w/ n == 4*m+p
+			} else {
+				result -= term;  // sign: I^n w/ n == 4*m+2+p
+			}
+		}
 		return result;
 	}
-	
-	ex a=basis.real_part();
-	ex b=basis.imag_part();
-	ex c=exponent.real_part();
-	ex d=exponent.imag_part();
-	return
-		power(abs(basis),c)*exp(-d*atan2(b,a))*sin(c*atan2(b,a)+d*log(abs(basis)));
-}
 
-// protected
+	// Im((a+I*b)^(c+I*d))
+	const ex d = exponent.imag_part();
+	return power(abs(basis),c)*exp(-d*atan2(b,a))*sin(c*atan2(b,a)+d*log(abs(basis)));
+}
 
 // protected
 
@@ -764,7 +802,7 @@ unsigned power::return_type() const
 	return basis.return_type();
 }
 
-tinfo_t power::return_type_tinfo() const
+return_type_t power::return_type_tinfo() const
 {
 	return basis.return_type_tinfo();
 }
@@ -777,6 +815,51 @@ ex power::expand(unsigned options) const
 		return *this;
 	}
 
+	// (x*p)^c -> x^c * p^c, if p>0
+	// makes sense before expanding the basis
+	if (is_exactly_a<mul>(basis) && !basis.info(info_flags::indefinite)) {
+		const mul &m = ex_to<mul>(basis);
+		exvector prodseq;
+		epvector powseq;
+		prodseq.reserve(m.seq.size() + 1);
+		powseq.reserve(m.seq.size() + 1);
+		epvector::const_iterator last = m.seq.end();
+		epvector::const_iterator cit = m.seq.begin();
+		bool possign = true;
+
+		// search for positive/negative factors
+		while (cit!=last) {
+			ex e=m.recombine_pair_to_ex(*cit);
+			if (e.info(info_flags::positive))
+				prodseq.push_back(pow(e, exponent).expand(options));
+			else if (e.info(info_flags::negative)) {
+				prodseq.push_back(pow(-e, exponent).expand(options));
+				possign = !possign;
+			} else
+				powseq.push_back(*cit);
+			++cit;
+		}
+
+		// take care on the numeric coefficient
+		ex coeff=(possign? _ex1 : _ex_1);
+		if (m.overall_coeff.info(info_flags::positive) && m.overall_coeff != _ex1)
+			prodseq.push_back(power(m.overall_coeff, exponent));
+		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
+			prodseq.push_back(power(-m.overall_coeff, exponent));
+		else
+			coeff *= m.overall_coeff;
+
+		// If positive/negative factors are found, then extract them.
+		// In either case we set a flag to avoid the second run on a part
+		// which does not have positive/negative terms.
+		if (prodseq.size() > 0) {
+			ex newbasis = coeff*mul(powseq);
+			ex_to<basic>(newbasis).setflag(status_flags::purely_indefinite);
+			return ((new mul(prodseq))->setflag(status_flags::dynallocated)*(new power(newbasis, exponent))->setflag(status_flags::dynallocated).expand(options)).expand(options);
+		} else
+			ex_to<basic>(basis).setflag(status_flags::purely_indefinite);
+	}
+
 	const ex expanded_basis = basis.expand(options);
 	const ex expanded_exponent = exponent.expand(options);
 	
@@ -846,90 +929,331 @@ ex power::expand(unsigned options) const
 // non-virtual functions in this class
 //////////
 
+namespace {  // anonymous namespace for power::expand_add() helpers
+
+/** Helper class to generate all bounded combinatorial partitions of an integer
+ *  n with exactly m parts (including zero parts) in non-decreasing order.
+ */
+class partition_generator {
+private:
+	// Partitions n into m parts, not including zero parts.
+	// (Cf. OEIS sequence A008284; implementation adapted from Jörg Arndt's
+	// FXT library)
+	struct mpartition2
+	{
+		// partition: x[1] + x[2] + ... + x[m] = n and sentinel x[0] == 0
+		std::vector<int> x;
+		int n;   // n>0
+		int m;   // 0<m<=n
+		mpartition2(unsigned n_, unsigned m_)
+		  : x(m_+1), n(n_), m(m_)
+		{
+			for (int k=1; k<m; ++k)
+				x[k] = 1;
+			x[m] = n - m + 1;
+		}
+		bool next_partition()
+		{
+			int u = x[m];  // last element
+			int k = m;
+			int s = u;
+			while (--k) {
+				s += x[k];
+				if (x[k] + 2 <= u)
+					break;
+			}
+			if (k==0)
+				return false;  // current is last
+			int f = x[k] + 1;
+			while (k < m) {
+				x[k] = f;
+				s -= f;
+				++k;
+			}
+			x[m] = s;
+			return true;
+		}
+	} mpgen;
+	int m;  // number of parts 0<m<=n
+	mutable std::vector<int> partition;  // current partition
+public:
+	partition_generator(unsigned n_, unsigned m_)
+	  : mpgen(n_, 1), m(m_), partition(m_)
+	{ }
+	// returns current partition in non-decreasing order, padded with zeros
+	const std::vector<int>& current() const
+	{
+		for (int i = 0; i < m - mpgen.m; ++i)
+			partition[i] = 0;  // pad with zeros
+
+		for (int i = m - mpgen.m; i < m; ++i)
+			partition[i] = mpgen.x[i - m + mpgen.m + 1];
+
+		return partition;
+	}
+	bool next()
+	{
+		if (!mpgen.next_partition()) {
+			if (mpgen.m == m || mpgen.m == mpgen.n)
+				return false;  // current is last
+			// increment number of parts
+			mpgen = mpartition2(mpgen.n, mpgen.m + 1);
+		}
+		return true;
+	}
+};
+
+/** Helper class to generate all compositions of a partition of an integer n,
+ *  starting with the compositions which has non-decreasing order.
+ */
+class composition_generator {
+private:
+	// Generates all distinct permutations of a multiset.
+	// (Based on Aaron Williams' algorithm 1 from "Loopless Generation of
+	// Multiset Permutations using a Constant Number of Variables by Prefix
+	// Shifts." <http://webhome.csc.uvic.ca/~haron/CoolMulti.pdf>)
+	struct coolmulti {
+		// element of singly linked list
+		struct element {
+			int value;
+			element* next;
+			element(int val, element* n)
+			  : value(val), next(n) {}
+			~element()
+			{   // recurses down to the end of the singly linked list
+				delete next;
+			}
+		};
+		element *head, *i, *after_i;
+		// NB: Partition must be sorted in non-decreasing order.
+		explicit coolmulti(const std::vector<int>& partition)
+		{
+			head = NULL;
+			for (unsigned n = 0; n < partition.size(); ++n) {
+				head = new element(partition[n], head);
+				if (n <= 1)
+					i = head;
+			}
+			after_i = i->next;
+		}
+		~coolmulti()
+		{   // deletes singly linked list
+			delete head;
+		}
+		void next_permutation()
+		{
+			element *before_k;
+			if (after_i->next != NULL && i->value >= after_i->next->value)
+				before_k = after_i;
+			else
+				before_k = i;
+			element *k = before_k->next;
+			before_k->next = k->next;
+			k->next = head;
+			if (k->value < head->value)
+				i = k;
+			after_i = i->next;
+			head = k;
+		}
+		bool finished() const
+		{
+			return after_i->next == NULL && after_i->value >= head->value;
+		}
+	} cmgen;
+	bool atend;  // needed for simplifying iteration over permutations
+	bool trivial;  // likewise, true if all elements are equal
+	mutable std::vector<int> composition;  // current compositions
+public:
+	explicit composition_generator(const std::vector<int>& partition)
+	  : cmgen(partition), atend(false), trivial(true), composition(partition.size())
+	{
+		for (unsigned i=1; i<partition.size(); ++i)
+			trivial = trivial && (partition[0] == partition[i]);
+	}
+	const std::vector<int>& current() const
+	{
+		coolmulti::element* it = cmgen.head;
+		size_t i = 0;
+		while (it != NULL) {
+			composition[i] = it->value;
+			it = it->next;
+			++i;
+		}
+		return composition;
+	}
+	bool next()
+	{
+		// This ugly contortion is needed because the original coolmulti
+		// algorithm requires code duplication of the payload procedure,
+		// one before the loop and one inside it.
+		if (trivial || atend)
+			return false;
+		cmgen.next_permutation();
+		atend = cmgen.finished();
+		return true;
+	}
+};
+
+/** Helper function to compute the multinomial coefficient n!/(p1!*p2!*...*pk!)
+ *  where n = p1+p2+...+pk, i.e. p is a partition of n.
+ */
+const numeric
+multinomial_coefficient(const std::vector<int> & p)
+{
+	numeric n = 0, d = 1;
+	std::vector<int>::const_iterator it = p.begin(), itend = p.end();
+	while (it != itend) {
+		n += numeric(*it);
+		d *= factorial(numeric(*it));
+		++it;
+	}
+	return factorial(numeric(n)) / d;
+}
+
+}  // anonymous namespace
+
 /** expand a^n where a is an add and n is a positive integer.
  *  @see power::expand */
 ex power::expand_add(const add & a, int n, unsigned options) const
 {
+	// The special case power(+(x,...y;x),2) can be optimized better.
 	if (n==2)
 		return expand_add_2(a, options);
 
-	const size_t m = a.nops();
-	exvector result;
+	// method:
+	//
+	// Consider base as the sum of all symbolic terms and the overall numeric
+	// coefficient and apply the binomial theorem:
+	// S = power(+(x,...,z;c),n)
+	//   = power(+(+(x,...,z;0);c),n)
+	//   = sum(binomial(n,k)*power(+(x,...,z;0),k)*c^(n-k), k=1..n) + c^n
+	// Then, apply the multinomial theorem to expand all power(+(x,...,z;0),k):
+	// The multinomial theorem is computed by an outer loop over all
+	// partitions of the exponent and an inner loop over all compositions of
+	// that partition. This method makes the expansion a combinatorial
+	// problem and allows us to directly construct the expanded sum and also
+	// to re-use the multinomial coefficients (since they depend only on the
+	// partition, not on the composition).
+	// 
+	// multinomial power(+(x,y,z;0),3) example:
+	// partition : compositions                : multinomial coefficient
+	// [0,0,3]   : [3,0,0],[0,3,0],[0,0,3]     : 3!/(3!*0!*0!) = 1
+	// [0,1,2]   : [2,1,0],[1,2,0],[2,0,1],... : 3!/(2!*1!*0!) = 3
+	// [1,1,1]   : [1,1,1]                     : 3!/(1!*1!*1!) = 6
+	//  =>  (x + y + z)^3 =
+	//        x^3 + y^3 + z^3
+	//      + 3*x^2*y + 3*x*y^2 + 3*y^2*z + 3*y*z^2 + 3*x*z^2 + 3*x^2*z
+	//      + 6*x*y*z
+	//
+	// multinomial power(+(x,y,z;0),4) example:
+	// partition : compositions                : multinomial coefficient
+	// [0,0,4]   : [4,0,0],[0,4,0],[0,0,4]     : 4!/(4!*0!*0!) = 1
+	// [0,1,3]   : [3,1,0],[1,3,0],[3,0,1],... : 4!/(3!*1!*0!) = 4
+	// [0,2,2]   : [2,2,0],[2,0,2],[0,2,2]     : 4!/(2!*2!*0!) = 6
+	// [1,1,2]   : [2,1,1],[1,2,1],[1,1,2]     : 4!/(2!*1!*1!) = 12
+	// (no [1,1,1,1] partition since it has too many parts)
+	//  =>  (x + y + z)^4 =
+	//        x^4 + y^4 + z^4
+	//      + 4*x^3*y + 4*x*y^3 + 4*y^3*z + 4*y*z^3 + 4*x*z^3 + 4*x^3*z
+	//      + 6*x^2*y^2 + 6*y^2*z^2 + 6*x^2*z^2
+	//      + 12*x^2*y*z + 12*x*y^2*z + 12*x*y*z^2
+	//
+	// Summary:
+	// r = 0
+	// for k from 0 to n:
+	//     f = c^(n-k)*binomial(n,k)
+	//     for p in all partitions of n with m parts (including zero parts):
+	//         h = f * multinomial coefficient of p
+	//         for c in all compositions of p:
+	//             t = 1
+	//             for e in all elements of c:
+	//                 t = t * a[e]^e
+	//             r = r + h*t
+	// return r
+
+	epvector result;
 	// The number of terms will be the number of combinatorial compositions,
 	// i.e. the number of unordered arrangements of m nonnegative integers
 	// which sum up to n.  It is frequently written as C_n(m) and directly
-	// related with binomial coefficients:
-	result.reserve(binomial(numeric(n+m-1), numeric(m-1)).to_int());
-	intvector k(m-1);
-	intvector k_cum(m-1); // k_cum[l]:=sum(i=0,l,k[l]);
-	intvector upper_limit(m-1);
-	int l;
-
-	for (size_t l=0; l<m-1; ++l) {
-		k[l] = 0;
-		k_cum[l] = 0;
-		upper_limit[l] = n;
+	// related with binomial coefficients: binomial(n+m-1,m-1).
+	size_t result_size = binomial(numeric(n+a.nops()-1), numeric(a.nops()-1)).to_int();
+	if (!a.overall_coeff.is_zero()) {
+		// the result's overall_coeff is one of the terms
+		--result_size;
 	}
+	result.reserve(result_size);
 
-	while (true) {
-		exvector term;
-		term.reserve(m+1);
-		for (l=0; l<m-1; ++l) {
-			const ex & b = a.op(l);
-			GINAC_ASSERT(!is_exactly_a<add>(b));
-			GINAC_ASSERT(!is_exactly_a<power>(b) ||
-			             !is_exactly_a<numeric>(ex_to<power>(b).exponent) ||
-			             !ex_to<numeric>(ex_to<power>(b).exponent).is_pos_integer() ||
-			             !is_exactly_a<add>(ex_to<power>(b).basis) ||
-			             !is_exactly_a<mul>(ex_to<power>(b).basis) ||
-			             !is_exactly_a<power>(ex_to<power>(b).basis));
-			if (is_exactly_a<mul>(b))
-				term.push_back(expand_mul(ex_to<mul>(b), numeric(k[l]), options, true));
-			else
-				term.push_back(power(b,k[l]));
+	// Iterate over all terms in binomial expansion of
+	// S = power(+(x,...,z;c),n)
+	//   = sum(binomial(n,k)*power(+(x,...,z;0),k)*c^(n-k), k=1..n) + c^n
+	for (int k = 1; k <= n; ++k) {
+		numeric binomial_coefficient;  // binomial(n,k)*c^(n-k)
+		if (a.overall_coeff.is_zero()) {
+			// degenerate case with zero overall_coeff:
+			// apply multinomial theorem directly to power(+(x,...z;0),n)
+			binomial_coefficient = 1;
+			if (k < n) {
+				continue;
+			}
+		} else {
+			binomial_coefficient = binomial(numeric(n), numeric(k)) * pow(ex_to<numeric>(a.overall_coeff), numeric(n-k));
 		}
 
-		const ex & b = a.op(l);
-		GINAC_ASSERT(!is_exactly_a<add>(b));
-		GINAC_ASSERT(!is_exactly_a<power>(b) ||
-		             !is_exactly_a<numeric>(ex_to<power>(b).exponent) ||
-		             !ex_to<numeric>(ex_to<power>(b).exponent).is_pos_integer() ||
-		             !is_exactly_a<add>(ex_to<power>(b).basis) ||
-		             !is_exactly_a<mul>(ex_to<power>(b).basis) ||
-		             !is_exactly_a<power>(ex_to<power>(b).basis));
-		if (is_exactly_a<mul>(b))
-			term.push_back(expand_mul(ex_to<mul>(b), numeric(n-k_cum[m-2]), options, true));
-		else
-			term.push_back(power(b,n-k_cum[m-2]));
+		// Multinomial expansion of power(+(x,...,z;0),k)*c^(n-k):
+		// Iterate over all partitions of k with exactly as many parts as
+		// there are symbolic terms in the basis (including zero parts).
+		partition_generator partitions(k, a.seq.size());
+		do {
+			const std::vector<int>& partition = partitions.current();
+			// All monomials of this partition have the same number of terms and the same coefficient.
+			const unsigned msize = count_if(partition.begin(), partition.end(), bind2nd(std::greater<int>(), 0));
+			const numeric coeff = multinomial_coefficient(partition) * binomial_coefficient;
 
-		numeric f = binomial(numeric(n),numeric(k[0]));
-		for (l=1; l<m-1; ++l)
-			f *= binomial(numeric(n-k_cum[l-1]),numeric(k[l]));
-
-		term.push_back(f);
-
-		result.push_back(ex((new mul(term))->setflag(status_flags::dynallocated)).expand(options));
-
-		// increment k[]
-		l = m-2;
-		while ((l>=0) && ((++k[l])>upper_limit[l])) {
-			k[l] = 0;
-			--l;
-		}
-		if (l<0) break;
-
-		// recalc k_cum[] and upper_limit[]
-		k_cum[l] = (l==0 ? k[0] : k_cum[l-1]+k[l]);
-
-		for (size_t i=l+1; i<m-1; ++i)
-			k_cum[i] = k_cum[i-1]+k[i];
-
-		for (size_t i=l+1; i<m-1; ++i)
-			upper_limit[i] = n-k_cum[i-1];
+			// Iterate over all compositions of the current partition.
+			composition_generator compositions(partition);
+			do {
+				const std::vector<int>& exponent = compositions.current();
+				exvector monomial;
+				monomial.reserve(msize);
+				numeric factor = coeff;
+				for (unsigned i = 0; i < exponent.size(); ++i) {
+					const ex & r = a.seq[i].rest;
+					GINAC_ASSERT(!is_exactly_a<add>(r));
+					GINAC_ASSERT(!is_exactly_a<power>(r) ||
+						     !is_exactly_a<numeric>(ex_to<power>(r).exponent) ||
+						     !ex_to<numeric>(ex_to<power>(r).exponent).is_pos_integer() ||
+						     !is_exactly_a<add>(ex_to<power>(r).basis) ||
+						     !is_exactly_a<mul>(ex_to<power>(r).basis) ||
+						     !is_exactly_a<power>(ex_to<power>(r).basis));
+					GINAC_ASSERT(is_exactly_a<numeric>(a.seq[i].coeff));
+					const numeric & c = ex_to<numeric>(a.seq[i].coeff);
+					if (exponent[i] == 0) {
+						// optimize away
+					} else if (exponent[i] == 1) {
+						// optimized
+						monomial.push_back(r);
+						if (c != *_num1_p)
+							factor = factor.mul(c);
+					} else { // general case exponent[i] > 1
+						monomial.push_back((new power(r, exponent[i]))->setflag(status_flags::dynallocated));
+						if (c != *_num1_p)
+							factor = factor.mul(c.power(exponent[i]));
+					}
+				}
+				result.push_back(a.combine_ex_with_coeff_to_pair(mul(monomial).expand(options), factor));
+			} while (compositions.next());
+		} while (partitions.next());
 	}
 
-	return (new add(result))->setflag(status_flags::dynallocated |
-	                                  status_flags::expanded);
+	GINAC_ASSERT(result.size() == result_size);
+
+	if (a.overall_coeff.is_zero()) {
+		return (new add(result))->setflag(status_flags::dynallocated |
+		                                  status_flags::expanded);
+	} else {
+		return (new add(result, ex_to<numeric>(a.overall_coeff).power(n)))->setflag(status_flags::dynallocated |
+		                                                                            status_flags::expanded);
+	}
 }
 
 
@@ -958,11 +1282,11 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		
 		if (c.is_equal(_ex1)) {
 			if (is_exactly_a<mul>(r)) {
-				sum.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                     _ex1));
+				sum.push_back(a.combine_ex_with_coeff_to_pair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
+				                                              _ex1));
 			} else {
-				sum.push_back(expair((new power(r,_ex2))->setflag(status_flags::dynallocated),
-				                     _ex1));
+				sum.push_back(a.combine_ex_with_coeff_to_pair((new power(r,_ex2))->setflag(status_flags::dynallocated),
+				                                              _ex1));
 			}
 		} else {
 			if (is_exactly_a<mul>(r)) {
@@ -977,14 +1301,14 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		for (epvector::const_iterator cit1=cit0+1; cit1!=last; ++cit1) {
 			const ex & r1 = cit1->rest;
 			const ex & c1 = cit1->coeff;
-			sum.push_back(a.combine_ex_with_coeff_to_pair((new mul(r,r1))->setflag(status_flags::dynallocated),
+			sum.push_back(a.combine_ex_with_coeff_to_pair(mul(r,r1).expand(options),
 			                                              _num2_p->mul(ex_to<numeric>(c)).mul_dyn(ex_to<numeric>(c1))));
 		}
 	}
 	
 	GINAC_ASSERT(sum.size()==(a.seq.size()*(a.seq.size()+1))/2);
 	
-	// second part: add terms coming from overall_factor (if != 0)
+	// second part: add terms coming from overall_coeff (if != 0)
 	if (!a.overall_coeff.is_zero()) {
 		epvector::const_iterator i = a.seq.begin(), end = a.seq.end();
 		while (i != end) {
@@ -1010,12 +1334,12 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 	}
 
 	// do not bother to rename indices if there are no any.
-	if ((!(options & expand_options::expand_rename_idx)) 
-			&& m.info(info_flags::has_indices))
+	if (!(options & expand_options::expand_rename_idx) &&
+	    m.info(info_flags::has_indices))
 		options |= expand_options::expand_rename_idx;
 	// Leave it to multiplication since dummy indices have to be renamed
 	if ((options & expand_options::expand_rename_idx) &&
-		(get_all_dummy_indices(m).size() > 0) && n.is_positive()) {
+	    (get_all_dummy_indices(m).size() > 0) && n.is_positive()) {
 		ex result = m;
 		exvector va = get_all_dummy_indices(m);
 		sort(va.begin(), va.end(), ex_is_less());
@@ -1050,4 +1374,6 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 	return result;
 }
 
+GINAC_BIND_UNARCHIVER(power);
+
 } // namespace GiNaC
